/*--------------------------------------------------------------------------
 * Copyright (C) 1992,1993,1994,1995,1996 Dallas Semiconductor Corporation. 
 * All rights Reserved. Printed in U.S.A.
 * This software is protected by copyright laws of
 * the United States and of foreign countries.
 * This material may also be protected by patent laws of the United States 
 * and of foreign countries.
 * This software is furnished under a license agreement and/or a
 * nondisclosure agreement and may only be used or copied in accordance
 * with the terms of those agreements.
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Dallas Semiconductor retains all ownership rights.
 *--------------------------------------------------------------------------
 */

/* includes */
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <dos.h>
#include <ctype.h>
#include <fcntl.h>
#include <io.h>
#include <time.h>
#include <windows.h>

/* type defs */
typedef unsigned char  uchar;
typedef unsigned short ushort;
/* typedef structure sent back by TMEX routines */
typedef struct      
{
        uchar name[4];
        uchar extension;
        uchar startpage;
        uchar numpages;
        uchar attrib;
        uchar bitmap[32];
} FileEntry;
/* structure to hold all device parameters by family code */
typedef struct
{
   uchar maxp;    /* max page number 0-255 */
   uchar memtype; /* 0 no mem , 1 nvram, 2 DS1982 type, 3 DS1985 and DS1986, 
                     4 DS2407, 5 eeprom DS1971 */
   char  *dsnum;  /* Dallas Semiconductor part number */
} Family;
typedef struct
{
   uchar NumEntries;      /* number of entries in path 0-10 */
   char  Ref;             /* reference character '\' or '.' */
   char  Entries[10][4];  /* sub-directory entry names */                                          
} DirectoryPath;

/* Error codes from TMEX functions */
#define NO_DEVICE             -1
#define WRONG_TYPE            -2
#define FILE_READ_ERR         -3
#define BUFFER_TOO_SMALL      -4
#define HANDLE_NOT_AVAIL      -5
#define FILE_NOT_FOUND        -6
#define REPEAT_FILE           -7
#define HANDLE_NOT_USED       -8
#define FILE_WRITE_ONLY       -9
#define OUT_OF_SPACE          -10
#define FILE_WRITE_ERR        -11
#define FILE_READ_ONLY        -12
#define FUNC_NOT_SUP          -13
#define BAD_FILENAME          -14
#define CANT_DEL_READ_ONLY    -15
#define HANDLE_NOT_EXIST      -16
#define ONE_WIRE_PORT_ERROR   -17
#define INVALID_DIRECTORY     -18
#define DIRECTORY_NOT_EMPTY   -19
#define UNABLE_TO_CREATE_DIR  -20
#define NO_PROGRAM_JOB        -21
#define PROGRAM_WRITE_PROTECT -22
#define NON_PROGRAM_PARTS     -23    
#define ADDFILE_TERMINATED    -24    /* (2.10) */
#define BAD_ARGUMENT           50
#define NO_DRIVERS             51
#define KEY_ABORT              52
#define OUT_MEMORY             53
#define NORMAL_EXIT            0

#define TRUE                   1
#define FALSE                  0
#define DIR_READ               1
#define DIR_SET                0
#define DIR_REMOVE             1
#define DIR_MAKE               0       
// for TMOneWireLevel
#define LEVEL_NORMAL           0
#define LEVEL_STRONG_PULLUP    1
#define LEVEL_BREAK            2
#define LEVEL_PROGRAM          3  
#define PRIMED_NONE            0
#define PRIMED_BIT             1
#define PRIMED_BYTE            2        
#define LEVEL_READ             1
#define LEVEL_SET              0

// session
extern long  far pascal TMExtendedStartSession(short, short, void far *);
extern short far pascal TMValidSession(long); 
extern short far pascal TMEndSession(long); 
extern short far pascal Get_Version(char far *); 
// file_operations
extern short far pascal TMFirstFile(long, void far *, FileEntry far *); 
extern short far pascal TMNextFile(long, void far *, FileEntry far *);
extern short far pascal TMOpenFile(long, void far *, FileEntry far *);
extern short far pascal TMCreateFile(long, void far *, short far *, FileEntry far *);
extern short far pascal TMCloseFile(long, void far *, short);
extern short far pascal TMReadFile(long, void far *, short, uchar far *, short);
extern short far pascal TMWriteFile(long, void far *, short, uchar far *, short);
extern short far pascal TMDeleteFile(long, void far *, FileEntry far *);
extern short far pascal TMFormat(long, void far *);
extern short far pascal TMAttribute(long, void far *, short, FileEntry far *);
extern short far pascal TMReNameFile(long, void far *, short, FileEntry far *); 
extern short far pascal TMChangeDirectory(long, void far *, short, DirectoryPath far *); 
extern short far pascal TMDirectoryMR(long, void far *, short, FileEntry far *);
extern short far pascal TMCreateProgramJob(long, void far *);
extern short far pascal TMDoProgramJob(long, void far *);
extern short far pascal TMWriteAddFile(long, void far *, short, short, short, uchar far *, short);
extern short far pascal TMTerminateAddFile(long, void far *, FileEntry far *);
// transport
extern short far pascal TMReadPacket(long, void far *, short, uchar far *, short);
extern short far pascal TMWritePacket(long, void far *, short, uchar far *, short);
extern short far pascal TMBlockIO(long, uchar far *, short);
extern short far pascal TMExtendedReadPage(long, void far *, short, uchar far *, short);
extern short far pascal TMProgramByte(long, void far *, short, short, short, short far *, short);
extern short far pascal TMProgramBlock(long, void far *, uchar far *, short, short, short far *);

// network
extern short far pascal TMSkipFamily(long, void far *); 
extern short far pascal TMFamilySearchSetup(long, void far *, short); 
extern short far pascal TMFirst(long, void far *);
extern short far pascal TMNext(long, void far *);
extern short far pascal TMAccess(long, void far *);
extern short far pascal TMOverAccess(long, void far *);
extern short far pascal TMStrongAccess(long, void far *);
extern short far pascal TMStrongAlarmAccess(long, void far *);
extern short far pascal TMRom(long, void far *, short far *);
extern short far pascal TMFirstAlarm(long, void far *);
extern short far pascal TMNextAlarm(long, void far *);  
extern short far pascal TMAutoOverDrive(long, void far *, short);  
// hardware_specific
extern short far pascal TMSetup(long);
extern short far pascal TMTouchByte(long, short);
extern short far pascal TMTouchReset(long);
extern short far pascal TMTouchBit(long, short);
extern short far pascal TMClose(long);
extern short far pascal TMProgramPulse(long); 
extern short far pascal TMOneWireCom(long, short, short); 
extern short far pascal TMOneWireLevel(long, short, short, short); 
extern short far pascal TMGetTypeVersion(short,char far *); 
extern short far pascal TMReadDefaultPort(short *, short *);


/* Global Variables */
#ifdef TMEXUTIL

/* specification for each device supported in tmex */
Family FamSpec[] = { {   0, 0, ""      }, // 00
                     {   0, 0, "1990"  }, // 01
                     {   0, 0, "1991"  }, // 02
                     {   0, 0, ""      }, // 03
                     {  15, 1, "1994"  }, // 04
                     {   0, 0, ""      }, // 05
                     {  15, 1, "1993"  }, // 06
                     {   0, 0, ""      }, // 07
                     {   3, 1, "1992"  }, // 08
                     {   3, 2, "1982"  }, // 09
                     {  63, 1, "1995"  }, // 0A
                     {  63, 3, "1985"  }, // 0B
                     { 255, 1, "1996"  }, // 0C
                     {   0, 0, ""      }, // 0D
                     {   0, 0, ""      }, // 0E
                     { 255, 3, "1986"  }, // 0F     
                     {   0, 0, ""      }, // 10
                     {   0, 0, ""      }, // 11
                     {   3, 4, "2407"  }, // 12  /* (2.10) */
                     {  15, 3, "2503"  }, // 13  /* (3.11Beta4) */
                     {   0, 5, "1971"  }, // 14  /* (2.10) */
                     {   0, 0, ""      }, // 15
                     {   0, 0, ""      }, // 16
                     {   0, 0, ""      }, // 17
                     {   0, 0, ""      }, // 18
                     {   0, 0, ""      }, // 19
                     {   0, 0, ""      }, // 1A
                     {   0, 0, ""      }, // 1B
                     {   0, 0, ""      }, // 1C
                     {   0, 0, ""      }, // 1D
                     {   0, 0, ""      }, // 1E
                     {   0, 0, ""      }, // 1F
                     {   0, 0, ""      }, // 20
                     {   0, 0, ""      }, // 21
                     {   0, 0, ""      }, // 22
                     {   0, 0, ""      }, // 23
                     {   0, 0, ""      }, // 24
                     {   0, 0, ""      }, // 25
                     {   0, 0, ""      }, // 26
                     {   0, 0, ""      }, // 27
                     {   0, 0, ""      }, // 28
                     {   0, 0, ""      }, // 29
                     {   0, 0, ""      }, // 2A
                     {   0, 0, ""      }, // 2B
                     {   0, 0, ""      }, // 2C
                     {   0, 0, ""      }, // 2D
                     {   0, 0, ""      }, // 2E
                     {   0, 0, ""      }, // 2F
                     {   0, 0, ""      }, // 30
                     {   0, 0, ""      }, // 31
                     {   0, 0, ""      }, // 32
                     {   0, 0, ""      }, // 33
                     {   0, 0, ""      }, // 34
                     {   0, 0, ""      }, // 35
                     {   0, 0, ""      }, // 36
                     {   0, 0, ""      }, // 37
                     {   0, 0, ""      }, // 38
                     {   0, 0, ""      }, // 39
                     {   0, 0, ""      }, // 3A
                     {   0, 0, ""      }, // 3B
                     {   0, 0, ""      }, // 3C
                     {   0, 0, ""      }, // 3D
                     {   0, 0, ""      }, // 3E
                     {   0, 0, ""      }, // 3F
                     {   0, 0, ""      }, // 40
                     {   0, 0, ""      }, // 41
                     {   0, 0, ""      }, // 42
                     {   0, 0, ""      }, // 43
                     {   0, 0, ""      }, // 44
                     {   0, 0, ""      }, // 45
                     {   0, 0, ""      }, // 46
                     {   0, 0, ""      }, // 47
                     {   0, 0, ""      }, // 48
                     {   0, 0, ""      }, // 49
                     {   0, 0, ""      }, // 4A
                     {   0, 0, ""      }, // 4B
                     {   0, 0, ""      }, // 4C
                     {   0, 0, ""      }, // 4D
                     {   0, 0, ""      }, // 4E
                     {   0, 0, ""      }, // 4F
                     {   0, 0, ""      }, // 50
                     {   0, 0, ""      }, // 51
                     {   0, 0, ""      }, // 52
                     {   0, 0, ""      }, // 53
                     {   0, 0, ""      }, // 54
                     {   0, 0, ""      }, // 55
                     {   0, 0, ""      }, // 56
                     {   0, 0, ""      }, // 57
                     {   0, 0, ""      }, // 58
                     {   0, 0, ""      }, // 59
                     {   0, 0, ""      }, // 5A
                     {   0, 0, ""      }, // 5B
                     {   0, 0, ""      }, // 5C
                     {   0, 0, ""      }, // 5D
                     {   0, 0, ""      }, // 5E
                     {   0, 0, ""      }, // 5F
                     {   0, 0, ""      }, // 60
                     {   0, 0, ""      }, // 61
                     {   0, 0, ""      }, // 62
                     {   0, 0, ""      }, // 63
                     {   0, 0, ""      }, // 64
                     {   0, 0, ""      }, // 65
                     {   0, 0, ""      }, // 66
                     {   0, 0, ""      }, // 67
                     {   0, 0, ""      }, // 68
                     {   0, 0, ""      }, // 69
                     {   0, 0, ""      }, // 6A
                     {   0, 0, ""      }, // 6B
                     {   0, 0, ""      }, // 6C
                     {   0, 0, ""      }, // 6D
                     {   0, 0, ""      }, // 6E
                     {   0, 0, ""      }, // 6F
                     {   0, 0, ""      }, // 70
                     {   0, 0, ""      }, // 71
                     {   0, 0, ""      }, // 72
                     {   0, 0, ""      }, // 73
                     {   0, 0, ""      }, // 74
                     {   0, 0, ""      }, // 75
                     {   0, 0, ""      }, // 76
                     {   0, 0, ""      }, // 77
                     {   0, 0, ""      }, // 78
                     {   0, 0, ""      }, // 79
                     {   0, 0, ""      }, // 7A
                     {   0, 0, ""      }, // 7B
                     {   0, 0, ""      }, // 7C
                     {   0, 0, ""      }, // 7D
                     {   0, 0, ""      }, // 7E
                     {   0, 0, ""      }  // 7F            
                  };

#else

extern Family FamSpec[128];  /* family specification values */

#endif
